library(kernplus)
library(gss)
library(FNN)
library(BayesTree)
source("amkFunctionLib.r")
args = commandArgs(TRUE)
if (args[1] == "WT5" | args[1] == "WT6"){type = "Offshore"} else {type = "Inland"}
data= read.csv(paste0(type," Wind Farm Dataset1(",args[1],").csv"))
data = data[,-1]
colnames(data)[ncol(data)] = "Power"
set.seed(10)
cv_sample = sample(nrow(data))
folds = cut(seq(1:length(cv_sample)),breaks = 5, labels = F)
CV_Err = matrix(NA,nrow = 5, ncol = 4) 
knn_data = matrix(NA,nrow = nrow(data),ncol = ncol(data))
for (i in 1:(ncol(data)-1)){
  min = min(data[,i])
  max = max(data[,i])
  knn_data[,i] = (data[,i] - min)/(max-min)
}
knn_data[,ncol(data)] = data[,ncol(data)]
k = seq(5,30,5)
for (f in 1:5){
  testindex = cv_sample[which(folds == f,arr.ind = T)]
  train_data = data[-testindex,]
  test_data = data[testindex,]
  knn_train_data = knn_data[-testindex,]
  knn_test_data = knn_data[testindex,]
  
  if (args[1] == "WT1" | args[1] == "WT3"){
    covariates = c(1,2,3,5)
    ssanova.fit = ssanova(Power~V*D*(air.density+S_b)-D:air.density-D:S_b, data = train_data, skip.iter = F)
  } else if (args[1] == "WT2" | args[1] == "WT4"){
    covariates = c(1:4)
    ssanova.fit = ssanova(Power~V*D*(air.density+I)-D:air.density-D:I, data = train_data, skip.iter = F)
  } else {
    covariates = c(1:4)
    ssanova.fit = ssanova(Power~V*D*(air.density+humidity)-D:air.density-D:humidity, data = train_data, skip.iter = F)
  }
  best.k = computeOptimalK(knn_train_data[,covariates],knn_train_data[,ncol(data)],CV.folds = 5, range.k = k)
  y_pred_knn = knn.reg(y = knn_train_data[,ncol(knn_data)], train = knn_train_data[,covariates],test = knn_test_data[,covariates], k = best.k)$pred
  CV_Err[f,1] = sqrt(mean((test_data$Power-y_pred_knn)^2))
  
  y_pred_ssanova = try(predict(ssanova.fit, test_data[,covariates]),silent = T)
  removeRow = NULL
  if (class(y_pred_ssanova) == "try-error"){
    y_pred_ssanova = NULL
    for (j in 1:nrow(test_data)){
      y_pred_ssanova = try(expr = predict(ssanova.fit,test_data[j,covariates]), silent = T)
      if (class(y_pred_ssanova)=="try-error"){
        removeRow = c(removeRow,j)
      }
    }
  }
  if (length(removeRow)==0){
    y_pred_ssanova = predict(ssanova.fit, test_data[,covariates])
    CV_Err[f,2] = sqrt(mean((test_data$Power-y_pred_ssanova)^2))
  } else {
    y_pred_ssanova = predict(ssanova.fit, test_data[-removeRow,covariates])
    CV_Err[f,2] = sqrt(mean((test_data$Power[-removeRow]-y_pred_ssanova)^2))
  }
  

  BART.fit = bart(x.train = train_data[,covariates], y.train = train_data[,ncol(train_data)], x.test = test_data[,covariates], verbose = F)
  y_pred_BART =  BART.fit$yhat.test.mean
  CV_Err[f,3] = sqrt(mean((test_data$Power-y_pred_BART)^2))
  
  
  y_pred_AMK = kp.pwcurv(y = train_data$Power, x = train_data[,covariates],x.new = test_data[,covariates],id.spd =1, id.dir = 2)
  removeRow = NULL
  removeRow = which(is.na(y_pred_AMK)==T)
  if (length(removeRow)!=0){
    CV_Err[f,4] = sqrt(mean((test_data$Power[-removeRow]-y_pred_AMK[-removeRow])^2))
  } else {
    CV_Err[f,4] = sqrt(mean((test_data$Power-y_pred_AMK)^2))
  }
}
RMSE = colMeans(CV_Err)
cat("   ","kNN","SSANOVA","BART","AMK","\n")
cat(args[1],round(RMSE,2),"\n")
